
function V=doublej_mp(G,D,H);

%
%  Computes the matrix V defined by the infinite sum
%
%      V = SUM   [G'^j]*D*[H^j]
%
%  where the sum goes from j = 0 to j = infinity.
%  The following doubling algorithm is used.
%  Starting from V(0) = D, iterate to convergence of V(j) on
%
%       G(j) = G(j-1)*G(j-1)
%       H(j) = H(j-1)*H(j-1)
%
%       V(j) = V(j-1) + G'(j-1)*V(j-1)*H(j-1)
%
%  The limit point is returned in V.

G0=G;
H0=H;
V0=D;

diff=5;

while diff > 1e-15;

  G1=G0*G0;
  H1=H0*H0;

  V1 = V0 + G0'*V0*H0;
  diff   = max(max(abs(V1-V0)));
  
  G0=G1;
  H0=H1;
  V0=V1;
  
end;

V = V1;