
function [A0,A1,A2,A3,A3T,A4,A4T,A5,Q,W,Q1,W1,Q2,W2,nbeta] = get_model_GK

global macropru_tool delegation kappa1 kappa2

% macropru_tool == 1 means funding subsidy for the bank
% macropru_tool == 2 means deposit subsidy for household
% macropru_tool == 3 means net worth injection into bank
% macropru_tool == 4 means reducing cost of liquidation

% delegation 1 : both social welfare objective
% delegation 2 : referee delegation 
% delegation 3 : monetary: gap and inflation; macropu :gap and effective rate
% delegation 4 : is 3 with Rogoff 


% kappa1 is relative weight on the gap for monetary authority
% kappa2 is relative weight on the gap for macroprudential authority

stdv_tec = 0.0023;
stdv_mas = 0.0043;
stdv_nws = 0.0412;
stdv_lam = 0.112;


chi     = 0.47;      
sigma   = 1;
theta   = chi;
w_sub   = 1;
nbeta   = 0.99;   
alpha   = 0.5;   
rho_tec = 0.95;   
rho_nws = 0;
rho_lam = 0.95;
rho_mas = 0.95;
epsi    = 11;   
ps      = epsi/(epsi-1);
fix_p   = 0.8;
slope_c = (1-fix_p)*(1-fix_p*nbeta)/fix_p;
% note that slope_r = epsi/adj, so that ...
adj     = epsi/slope_c;     
omega   = 0.0;
tau     = 1.5;

TAX_SS  = 1;
NOI_SS = 1/nbeta;

PHI_SS = 0.02/4;        % 2 percent spread annualized
LEV_SS = 1/0.11;         % consistent with 20% capital requirement
gamma  = (1-omega*LEV_SS)*nbeta/(PHI_SS*LEV_SS+1);
GRN_SS = 1/nbeta*(PHI_SS*LEV_SS+1);
GRB_SS = GRN_SS;
VAN_SS = (1-gamma)/(1-gamma*nbeta*GRN_SS);
VAB_SS = PHI_SS*VAN_SS;
LAM_SS = ((VAN_SS)/LEV_SS + VAB_SS)/TAX_SS;

% labor to utilization ratio in steady state
LUR_SS = (alpha/(1-alpha)*nbeta/(1+PHI_SS))^(1/(1+chi));
UTI_SS = ( (1-alpha)*LUR_SS^(alpha*(1-sigma)) )^(1/(theta+sigma));
OUT_SS = LUR_SS^alpha*UTI_SS;
LAB_SS = LUR_SS*UTI_SS;

CON_SS = OUT_SS;
WAG_SS = LAB_SS^chi*CON_SS^sigma;
LOA_SS = WAG_SS*LAB_SS;
NTW_SS = LOA_SS/LEV_SS;
PIN_SS = 1;
MAC_SS = 1;
PRI_SS = 1;
EXX_SS = OUT_SS;
TEC_SS = 0;

kappa  = slope_c; 
SUBb_SS = 1;    

MAS     =   	1;
NWS 	= MAS + 1;
TEC 	= MAS + 2;
LAM 	= MAS + 3;
PRL	    = MAS + 4;
WAG 	= MAS + 5;
OUT 	= MAS + 6;
LAB 	= MAS + 7;
INF 	= MAS + 8;
SUM 	= MAS + 9;
MAC 	= MAS + 10;
PRI 	= MAS + 11;
PHI 	= MAS + 12;
GAP 	= MAS + 13;
LAB2    = MAS + 14;
NOI_Y   = MAS + 15;
GAPcb   = MAS + 16;
GAPmp   = MAS + 17;
DIS 	= MAS + 18;
LOA 	= MAS + 19;
LEV 	= MAS + 20;
VAB   	= MAS + 21;
VAN   	= MAS + 22;
GRN 	= MAS + 23;
GRB 	= MAS + 24;
NTW 	= MAS + 25;
NOI     = MAS + 26;
SUBb_Y  = MAS + 27;   % NOTE: subsidy must be last for ONE_INSTRUMENT code to work

NOIA    = 1;
SUBb    = 1;

nz   = MAS + 27;  % non-policy variables
nu1  = 1;         % policy variables policymaker
nu2  = 1;  
ns   = MAS + 3;   % i.i.d shocks

A0  = zeros(nz,nz);
A1  = zeros(nz,nz);
A2  = zeros(nz,nz);
A3  = zeros(nz,nu1);
A3T = zeros(nz,nu2);
A4  = zeros(nz,nu1);
A4T = zeros(nz,nu2);
A5  = zeros(nz,ns);

% where: Y is non policy variables
%        X is policy variables
%        V is pure i.i.d. shocks
%
% A0*Y_{t} = A1*Y_{t-1} + A2*Y_{t+1} + A3*X1_t + A4*X1_{t+1} + A3T*X2_t + A4T*X2_{t+1} +A5*V_t

e = 1;
% 1. WAG  = sigma*OUT + theta*LAB;
A0(e,WAG) = 1;
A0(e,OUT) = -sigma;
A0(e,LAB) = -theta;

e = e+1;
% 2.  sigma*(OUT(+1) - OUT) = NOIA - PIN(+1);
A3(e,NOIA)=  1;
if macropru_tool == 2
   A3T(e,SUBb) = 1;    
end
A2(e,INF) = -1;
A2(e,OUT) = -sigma;
A0(e,OUT) = -sigma;

e=e+1;
% 3. OUT + MAC = NOI + WAG + LAB + (PHI_SS/(1+PHI_SS))*PHI;
A0(e,OUT) = 1;
A0(e,MAC) = 1;
A0(e,NOI) = -1;
A0(e,WAG) = -1;
A0(e,LAB) = -1;
A0(e,PHI) = -1/(1+PHI_SS);

e = e+1;
% 4. MAC = PRI - TEC;
A0(e,MAC) =  1;
A0(e,PRI) = -1;
A0(e,TEC) =  1;

e = e+1;
% 5. % INF = kappa*MAC + beta*INF(+1) + kappa*MAS ;
A0(e,INF)  = 1;
A2(e,INF)  =  nbeta;
A0(e,MAC)  = -kappa;
if MAS == 1;
   A0(e,MAS)  = -kappa;
end;

e=e+1;
% 6. OUT - LAB2 + MAC = sigma*OUT + theta*LAB2
A0(e,OUT)  = 1-sigma;
A0(e,LAB2) = -1 - theta;
A0(e,MAC)  = 1;

e=e+1;
% 7. OUT = TEC + alpha*LAB + (1-alpha)*LAB2; 
A0(e,OUT) = 1;
A0(e,TEC) = -1;
A0(e,LAB) = -alpha;
A0(e,LAB2)= -(1-alpha);

%EQUATIONS FROM BANKS
e=e+1;
% 8. Discount factor 
% DIS = CON^(-sigma)*CON(-1)^sigma/PIN;
A0(e,DIS) = 1;
A0(e,OUT) = sigma;
A1(e,OUT) = sigma;
A0(e,INF) = 1;

e=e+1;
% 9. borrowing constraint    
A0(e,LOA) = 1;
A0(e,PRL) = -1;
A0(e,WAG) = -1;
A0(e,LAB) = -1;

e=e+1;
% 10. LEV = LOA-NTW
A0(e,LEV) = 1;
A0(e,LOA) = -1;
A0(e,NTW) = 1;

e=e+1; 
% 11. NTW EQUATION
% NTW = gamma*(PHI(-1)*LEV(-1) + 1)*NOI(-1)*NTW(-1)*exp(NWS) + omega*LOA(-1) ;
A0(e,NTW) =  NTW_SS;
A0(e,NWS) = -NTW_SS;
A1(e,NTW) = gamma*(PHI_SS*LEV_SS + 1)*NOI_SS*NTW_SS;
A1(e,NOI) = gamma*(PHI_SS*LEV_SS + 1)*NOI_SS*NTW_SS;
A1(e,PHI) = gamma*LEV_SS*NOI_SS*NTW_SS;
A1(e,LEV) = gamma*PHI_SS*LEV_SS*NOI_SS*NTW_SS;
if macropru_tool == 3
   A3T(e,SUBb) = NTW_SS;
end


e=e+1;
% 12. running away constraint 
% VAN = (LAM - VAB)*LEV 
A0(e,VAN) =  VAN_SS;
A0(e,LAM) = -LEV_SS*LAM_SS;
A0(e,VAB) =  LEV_SS*VAB_SS;
A0(e,LEV) = -VAN_SS;
if macropru_tool == 4
   A3T(e,SUBb) = LEV_SS*LAM_SS;
end


e=e+1;
% 13 VAB   
% VAB = (1-gamma)*PHI + nbeta*gamma*DIS(+1)*GRB(+1)*VAB(+1);
A0(e,VAB) = VAB_SS;
A0(e,PHI) = -(1-gamma);
A2(e,GRB) = gamma*nbeta*VAB_SS*GRB_SS;
A2(e,DIS) = gamma*nbeta*VAB_SS*GRB_SS;
A2(e,VAB) = gamma*nbeta*VAB_SS*GRB_SS;


e=e+1;
% 14. VAN   
% VAN = (1-gamma)     + nbeta*gamma*DIS(+1)*GRN(+1)*VAN(+1);
A0(e,VAN) = VAN_SS;
A2(e,GRN) = gamma*nbeta*VAN_SS*GRN_SS;
A2(e,DIS) = gamma*nbeta*VAN_SS*GRN_SS;
A2(e,VAN) = gamma*nbeta*VAN_SS*GRN_SS;

e=e+1;
% 15.   GRN
% GRN = (LEV(-1)*PHI(-1) + 1)*NOI(-1);
A0(e,GRN)   = GRN_SS;
A1(e,NOI)   = GRN_SS;
A1(e,LEV)   = LEV_SS*PHI_SS*NOI_SS;
A1(e,PHI)   = LEV_SS*NOI_SS;

e=e+1;
% 16.   GRB 
%    GRB = LEV/LEV(-1)*GRN;
A0(e,GRB) = 1;
A0(e,LEV) = -1;
A1(e,LEV) = -1;
A0(e,GRN) = -1;

e=e+1;
% 17.   Price level
%A0(e,INF) = 1;
A0(e,PRL) = -1;
%A1(e,PRL) = -1;

%EQUATIONS FOR SHOCKS

e=e+1;
% 18. Law of motion for TFP;
A0(e,TEC) = 1;
A1(e,TEC) = rho_tec;
A5(e,TEC) = stdv_tec;

e=e+1;
% 19. Law of motion for NTW;
A0(e,NWS) = 1;
A1(e,NWS) = rho_nws;
A5(e,NWS) = stdv_nws;
if MAS == 1;
  e=e+1;
  % 20. Law of motion for MAS;  
  A0(e,MAS) = 1;
  A1(e,MAS) = rho_mas;
  A5(e,MAS) = stdv_mas;
end;

e=e+1;
% 21. Law of motion for LAM;
A0(e,LAM) = 1;
A1(e,LAM) = rho_lam;
A5(e,LAM) = stdv_lam;

%EQUATIONS FOR INSTRUMENTS IN Y VECTOR

e= e+1;
% 22. Defines the nominal interest rate in the Y-vector
A0(e,NOI_Y) = 1;
A0(e,NOI)   = 1;

e=e+1;
% 23. Effective interest rate
A0(e,SUM)   = 1;
A0(e,PHI)   = -1/(1+PHI_SS);
A0(e,NOI)   = -1; 

e=e+1;
% 24. GAP = OUT - (1+theta)/(sigma+theta)*TEC; %TO BE CONFIRMED
A0(e,GAP) = 1;
A0(e,OUT) = -1;
A0(e,TEC) = (1+theta)/(sigma+theta);

e= e+1;
% 25. Defines central bank's output gap
A0(e,GAPcb) = 1;
A0(e,INF)   = -1/kappa;
A2(e,INF)   = -nbeta/kappa;
if MAS == 1;
    A0(e,MAS)   = 1;
end;
A0(e,NOI)   =  -alpha;

e= e+1;
% 26. Defines macropru's output gap
A0(e,GAPmp) = 1;
A0(e,SUBb)  = 1;
A0(e,PHI)   = 1/PHI_SS;

% 27. Defines NOIA  = NOI + SUBb
e=e+1;
A3(e,NOIA) = -1;
A0(e,NOI ) = -1;
if macropru_tool == 1;
   A3T(e,SUBb)=  1;
end

e= e+1;
% 28. Defines the sub in the Y-vector  - MUST BE LAST EQUATION, for one instrument code to work !
A0(e,SUBb_Y) = 1;
A3T(e,SUBb)  = 1;


% loss function
% ------------------------------
Q = zeros(nu1+nu2,nu1+nu2);
Q(2,2) = 0;

W = zeros(nz,nz);

W1 = W; 
W2 = W;

Q1 = zeros(nu1,nu1);
Q2 = Q(2,2); % zeros(nu2,nu2);


W(INF,INF)     = adj;
W(GAP,GAP)     = (sigma+theta);
W(SUM,SUM)     = alpha*(1-alpha)/(1+theta);
 

if delegation == 1;
    W1 = W;
    W2 = W;
elseif delegation ==2;
    W1(INF,INF) = adj;
    W1(GAP,GAP) = (sigma+theta);
    W2(SUM,SUM) = alpha*(1-alpha)/(1+theta);
elseif delegation ==  3;
    W1(INF,INF) = adj;
    W1(GAP,GAP) = (sigma+theta);
    W2(GAP,GAP) = (sigma+theta);
    W2(SUM,SUM) = alpha*(1-alpha)/(1+theta);
elseif delegation == 4;
    W1(INF,INF) = adj;
    W1(GAP,GAP) = kappa1*(sigma+theta);
    W2(GAP,GAP) = kappa2*(sigma+theta);
    W2(SUM,SUM) = alpha*(1-alpha)/(1+theta);
end;    
   