function [WL_benchmark,OW,WL_OW] = cooperation_WL()

%-------------------------------------------------------------------------------
% Solve for the benchmark welfare loss
%-------------------------------------------------------------------------------
WL_benchmark = cooperation_WL_calc(1,1,1,1);

%-------------------------------------------------------------------------------
% Solve for the optimal weight
%-------------------------------------------------------------------------------
grid_wgap = linspace(0.01,1,100);
grid_wf   = linspace(0.01,1,100);

N_wf      = length(grid_wf);
N_wgap    = length(grid_wgap);

%Loop over all the candidate weights
WL        = zeros(N_wf,N_wgap);
parfor i_wgap = 1:N_wgap
    WL_slice = zeros(N_wf,1);
    
    for i_wf = 1:N_wf
        WL_slice(i_wf) = cooperation_WL_calc(grid_wgap(i_wgap),grid_wf(i_wf),1,1);
    end
    
    WL(:,i_wgap) = WL_slice(:);
end

%Find the minimum weight
[min_WL,index] = min(WL(:));

%Convert the flat index into a grid index and get the weights
[i_wf,i_wgap] = ind2sub(size(WL),index);

%Produce the output
OW    = sprintf('(%1.2f, %1.2f)',grid_wgap(i_wgap),grid_wf(i_wf));
WL_OW = sprintf('%1.4f',min_WL);

end

function social_welfare_conditional = cooperation_WL_calc(wgap_cur,wf_cur,wgapCB_cur,wgapMP_cur)

%-------------------------------------------------------------------------------
% Solve the model
%-------------------------------------------------------------------------------
global wgap wf wgapCB wgapMP macropru_tool
wgap   = wgap_cur;
wf     = wf_cur;
wgapCB = wgapCB_cur;
wgapMP = wgapMP_cur;

macropru_tool = 2;

%Get the model matricies
[A0,A1,A2,A3,A3T,A4,A4T,A5,Q,W,WS,Q1,W1,Q2,W2,beta] = get_model_reduced_varWeight();
WC = W;
A3 = [A3,A3T];
A4 = [A4,A4T];

[nz,nu] = size(A3);
ns      = size(A5,2);

%Run the solver
t        = 1e-10; 
dampen   = 0;

F1_start = zeros(nu,nz); 
H1_start = zeros(nz,nz);
V_start  = zeros(nz,nz);

[H1,H2,F1,F2,V] = solve_value_coop(A0,A1,A2,A3,A4,A5,WC,Q,H1_start,F1_start,V_start,beta,t,dampen);

%-------------------------------------------------------------------------------
% Compute the welfare loss
%-------------------------------------------------------------------------------
OMEGA_U                    = zeros(ns,ns);
OMEGA_U(1,1)               = 1;

W = WS;

P0                         = doublej_mp(beta*H1,  W  + beta*F1'*Q*F1,  H1); 
social_welfare_conditional = 1/2*(trace(H2(:,1)'*P0*H2(:,1))*(1-beta) + beta*trace( (F2'*Q*F2  + H2'*P0*H2)*OMEGA_U))*100;

end