function [WL_benchmark,OW,WL_OW] = macroPrud_WL()

%-------------------------------------------------------------------------------
% Calculate the benchmark value
%-------------------------------------------------------------------------------
WL_benchmark = macroPrud_WL_calc(1,1);

%-------------------------------------------------------------------------------
% Calculate optimal weights
%-------------------------------------------------------------------------------
N_gapCB     = 100;
N_gapMP     = 100;

grid_wgapCB = linspace(0,1.5,N_gapCB);
grid_wgapMP = linspace(0,1.5,N_gapMP);

WL          = zeros(N_gapCB,N_gapMP);
parfor i_MP = 1:N_gapMP
    for i_CB = 1:N_gapCB
        WL(i_CB,i_MP) = macroPrud_WL_calc(grid_wgapCB(i_CB),grid_wgapMP(i_MP));
    end
end

%Find the minimum weight
[min_WL,index] = min(WL(:));

%Convert the flat index into a grid index and get the weights
[i_CB,i_MP] = ind2sub(size(WL),index);

%Produce the output
OW    = sprintf('(%1.2f, %1.2f)',grid_wgapCB(i_CB),grid_wgapMP(i_MP));
WL_OW = sprintf('%1.4f',min_WL);

end

function social_welfare_conditional = macroPrud_WL_calc(wgapCB_cur,wgapMP_cur)

%-------------------------------------------------------------------------------
% Solve the Model
%-------------------------------------------------------------------------------
global dampen solution_algo wgapCB wgapMP macropru_tool
dampen        = 0;
macropru_tool = 1;
solution_algo = 2;
wgapCB        = wgapCB_cur;
wgapMP        = wgapMP_cur;

%Get the Model matrix
[A0,A1,A2,A3,A3T,A4,A4T,A5,Q,W,WS,Q1,W1,Q2,W2,beta] = get_model_reduced_loop_Bianca;

WW = W;
QQ = zeros(size(Q));

[nz,nu1] = size(A3);
nu2      = size(A3T,2);
ns       = size(A5,2); 
 
%Call the solver
t   = 1e-10;
H1  = zeros(nz,nz);
F1  = zeros(1,nz);
F1T = zeros(1,nz);
G   = 0;

[H1,H2,F1,F1T,F2,F2T,G] = solve_leader(A0,A1,A2,A3T,A3,A4T,A4,A5,W2,W1,Q2,Q1,H1,F1T,F1,G,beta,t); 

%-------------------------------------------------------------------------------
% Calculate the Welfare loss
%-------------------------------------------------------------------------------
OMEGA_U      = zeros(ns,ns);
OMEGA_U(1,1) = 1;
P0           = doublej_mp(beta*H1,  WW  + beta*transpose([F1;F1T])*QQ*[F1;F1T],  H1);

social_welfare_conditional = 1/2*(trace(H2(:,1)'*P0*H2(:,1))*(1-beta) + beta*trace(([F2;F2T]'*QQ*[F2;F2T]  + H2'*P0*H2)*OMEGA_U))*100;
end