function [WL_benchmark,OW,WL_OW] = noMacroPrud_WL()

%-------------------------------------------------------------------------------
% Solve for the benchmark welfare loss
%-------------------------------------------------------------------------------
WL_benchmark = noMacroPrud_WL_calc(1,1,1,1);

%-------------------------------------------------------------------------------
% Solve for the optimal weight
%-------------------------------------------------------------------------------
grid_wgap = linspace(0,1,101);
grid_wf   = linspace(0,1,101);

N_wf      = length(grid_wf);
N_wgap    = length(grid_wgap);

%Loop over all the candidate weights
WL        = zeros(N_wf,N_wgap);
parfor i_wgap = 1:N_wgap
    WL_slice = zeros(N_wf,1);
    
    for i_wf = 1:N_wf
        WL_slice(i_wf) = noMacroPrud_WL_calc(grid_wgap(i_wgap),grid_wf(i_wf),1,1);
    end
    
    WL(:,i_wgap) = WL_slice(:);
end

%Find the minimum weight
[min_WL,index] = min(WL(:));

%Convert the flat index into a grid index and get the weights
[i_wf,i_wgap] = ind2sub(size(WL),index);

%Produce the output
OW    = sprintf('(%1.2f, %1.2f)',grid_wgap(i_wgap),grid_wf(i_wf));
WL_OW = sprintf('%1.4f',min_WL);

end

function social_welfare_conditional = noMacroPrud_WL_calc(wgap_cur,wf_cur,wgapCB_cur,wgapMP_cur)

%-------------------------------------------------------------------------------
% Solve the model
%-------------------------------------------------------------------------------
global wgap wf wgapCB wgapMP macropru_tool
wgap   = wgap_cur;
wf     = wf_cur;
wgapCB = wgapCB_cur;
wgapMP = wgapMP_cur;
macropru_tool = 1;

%Get the model matricies
[A0,A1,A2,A3,A3T,A4,A4T,A5,Q,W,WS,Q1,W1,Q2,W2,beta] = get_model_reduced_varWeight();

[nz,nu]  = size(A3);
nz       = nz - 1;
A0       = A0(1:nz,1:nz);
A1       = A1(1:nz,1:nz);
A2       = A2(1:nz,1:nz);
A3       = A3(1:nz,:);
A4       = A4(1:nz,:);
A5       = A5(1:nz,:);
W        = W(1:nz,1:nz);
WS       = WS(1:nz,1:nz);
Q        = Q(1,1);  
  
[nz,nu]  = size(A5);
[nz,ns1] = size(A3);
ns       = size(A5,2);  

%Solve the model
t        = 1e-10;     % convergence criterion 
F1_start = zeros(ns1,nz); 
H1_start = zeros(nz,nz);
V_start  = zeros(nz,nz);

[H1,H2,F1,F2] = solve_coop(A0,A1,A2,A3,A4,A5,W,Q,H1_start,F1_start,beta,t);

%-------------------------------------------------------------------------------
% Calculate the social welfare loss
%-------------------------------------------------------------------------------
OMEGA_U                    = zeros(ns,ns);
OMEGA_U(1,1)               = 1;

P0                         = doublej_mp(beta*H1,  WS  + beta*F1'*Q*F1,  H1); 

social_welfare_conditional = 1/2*(trace(H2(:,1)'*P0*H2(:,1))*(1-beta) + beta*trace( (F2'*Q*F2  + H2'*P0*H2)*OMEGA_U))*100;

end